@@ -7,13 +7,13 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: util.c 13520 2012-09-23 15:38:07Z jordan $
+ * $Id: util.c 13560 2012-10-13 16:47:26Z jordan $
  */
 
 #include <ctype.h> /* isxdigit() */
 #include <errno.h>
 #include <stdarg.h>
-#include <string.h> /* strchr(), strrchr(), strlen(), strncmp(), strstr() */
+#include <string.h> /* strchr(), strrchr(), strlen(), strstr() */
 
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -170,15 +170,17 @@ gtr_get_host_from_url( char * buf, size_t buflen, const char * url )
 static gboolean
 gtr_is_supported_url( const char * str )
 {
-    return !strncmp( str, "ftp://", 6 )
-        || !strncmp( str, "http://", 7 )
-        || !strncmp( str, "https://", 8 );
+    return ( ( str != NULL ) &&
+             ( g_str_has_prefix( str, "ftp://" ) ||
+               g_str_has_prefix( str, "http://" ) ||
+               g_str_has_prefix( str, "https://" ) ) );
 }
 
 gboolean
 gtr_is_magnet_link( const char * str )
 {
-    return !strncmp( str, "magnet:?", 8 );
+    return ( str != NULL ) &&
+           ( g_str_has_prefix( str, "magnet:?" ) );
 }
 
 gboolean
@@ -584,8 +586,8 @@ gtr_paste_clipboard_url_into_entry( GtkWidget * e )
   size_t i;
 
   char * text[] = {
-    gtk_clipboard_wait_for_text( gtk_clipboard_get( GDK_SELECTION_PRIMARY ) ),
-    gtk_clipboard_wait_for_text( gtk_clipboard_get( GDK_SELECTION_CLIPBOARD ) )
+    g_strstrip( gtk_clipboard_wait_for_text( gtk_clipboard_get( GDK_SELECTION_PRIMARY ) ) ),
+    g_strstrip( gtk_clipboard_wait_for_text( gtk_clipboard_get( GDK_SELECTION_CLIPBOARD ) ) )
   };
 
   for( i=0; i<G_N_ELEMENTS(text); ++i ) {
