@@ -1,10 +1,7 @@
-#include <stdio.h>
-#include <string.h>
 #include "transmission.h"
 #include "magnet.h"
 #include "utils.h"
 
-/* #define VERBOSE */
 #undef VERBOSE
 #include "libtransmission-test.h"
 
@@ -25,12 +22,12 @@ test1( void )
           "&ws=http%3A%2F%2Fserver.webseed.org%2Fpath%2Fto%2Ffile";
     info = tr_magnetParse( uri );
     check( info != NULL );
-    check( info->trackerCount == 2 );
-    check( !strcmp( info->trackers[0], "http://tracker.openbittorrent.com/announce" ) );
-    check( !strcmp( info->trackers[1], "http://tracker.opentracker.org/announce" ) );
-    check( info->webseedCount == 1 );
-    check( !strcmp( info->webseeds[0], "http://server.webseed.org/path/to/file" ) );
-    check( !strcmp( info->displayName, "Display Name" ) );
+    check_int_eq( 2, info->trackerCount );
+    check_streq( info->trackers[0], "http://tracker.openbittorrent.com/announce" );
+    check_streq( info->trackers[1], "http://tracker.opentracker.org/announce" );
+    check_int_eq( 1, info->webseedCount );
+    check_streq( "http://server.webseed.org/path/to/file", info->webseeds[0] );
+    check_streq( "Display Name", info->displayName );
     for( i=0; i<20; ++i )
         check( info->hash[i] == dec[i] );
     tr_magnetFree( info );
@@ -45,12 +42,12 @@ test1( void )
           "&tr=http%3A%2F%2Ftracker.opentracker.org%2Fannounce";
     info = tr_magnetParse( uri );
     check( info != NULL );
-    check( info->trackerCount == 2 );
-    check( !strcmp( info->trackers[0], "http://tracker.openbittorrent.com/announce" ) );
-    check( !strcmp( info->trackers[1], "http://tracker.opentracker.org/announce" ) );
-    check( info->webseedCount == 1 );
-    check( !strcmp( info->webseeds[0], "http://server.webseed.org/path/to/file" ) );
-    check( !strcmp( info->displayName, "Display Name" ) );
+    check_int_eq( 2, info->trackerCount );
+    check_streq( "http://tracker.openbittorrent.com/announce", info->trackers[0] );
+    check_streq( "http://tracker.opentracker.org/announce", info->trackers[1] );
+    check_int_eq( 1, info->webseedCount );
+    check_streq( "http://server.webseed.org/path/to/file", info->webseeds[0] );
+    check_streq( "Display Name", info->displayName );
     for( i=0; i<20; ++i )
         check( info->hash[i] == dec[i] );
     tr_magnetFree( info );
@@ -59,19 +56,5 @@ test1( void )
     return 0;
 }
 
-int
-main( void )
-{
-    const testFunc tests[] = { test1 };
-    int ret;
-
-    if( (ret = runTests(tests, 1)) )
-        return ret;
-
-#ifdef VERBOSE
-    fprintf( stderr, "magnet-test passed\n" );
-#endif
-
-    return 0;
-}
+MAIN_SINGLE_TEST(test1)
 
